from spynnaker7._version import __version__  # NOQA
from spynnaker7._version import __version_name__  # NOQA
from spynnaker7._version import __version_month__  # NOQA
from spynnaker7._version import __version_year__  # NOQA
